#include "fftplot.h"

#include <culinkstats.h>
#include <quplotcommon.h>
#include <cucontext.h>
#include <cucontrolsutils.h>
#include <quplotcurve.h>

class FFTPlotPrivate {
public:
    int num_turns;
    QuPlotCommon *plot_common;
    bool read_ok;
    CuControlsUtils u;
    QVector<double> x_data;

    void fill_x_data(int siz) {
        x_data.clear();
        for(int i = 0; i < siz; i++)
            x_data.push_back((i+1)/num_turns);
    }
};

FFTPlot::FFTPlot(QWidget* parent, CumbiaPool *cu_p, const CuControlsFactoryPool& fp)
    : QuSpectrumPlot(parent, cu_p, fp) {
    d = new FFTPlotPrivate;
    d->plot_common = new QuPlotCommon(cu_p, fp);
    d->num_turns = 1024;

}

FFTPlot::~FFTPlot()
{
    delete d->plot_common;
    delete d;
}

void FFTPlot::set_num_turns(double t) {
    d->num_turns = t;
}


void FFTPlot::onUpdate(const CuData &da) {
    d->read_ok = !da["err"].toBool();
    const CuVariant &v = da["value"];
    const QString &src = QString::fromStdString(da["src"].toString());
    const QString& msg = d->u.msg(da);

    // update link statistics
    CuLinkStats *link_s = d->plot_common->getContext()->getLinkStats();
    link_s->addOperation();
    if(!d->read_ok)
        link_s->addError(msg.toStdString());


    // configure triggers replot at the end but should not be too expensive
    // to do it once here at configuration time and once more from appendData
    if(d->read_ok && da["type"].toString() == std::string("property")) {
        configure(da);
    }

    QuPlotCurve *crv = curve(src);
    if(!crv) {
        addCurve(src, crv = new QuPlotCurve(src));
    }
    d->read_ok ? crv->setState(QuPlotCurve::Normal) : crv->setState(QuPlotCurve::Invalid);

    if(d->read_ok && v.isValid() && v.getFormat() == CuVariant::Vector)
    {
        std::vector<double> out;
        v.toVector<double>(out);
        QVector<double> y  = QVector<double>::fromStdVector(out);
        if(y.size() != d->x_data.size() && y.size() > 0) {
            d->fill_x_data(y.size());
            setXLowerBound(d->x_data.first());
            setXUpperBound(d->x_data.last());
            printf("FFTPlot.update: xl %f xu%f\n", xLowerBound(), xUpperBound());
        }
        setData(src, d->x_data, y);
    }
    else {
        // appendData triggers a replot when necessary. If !d->read_ok, then there's at least
        // one curve with an Invalid state. A replot is necessary in this case
        replot();
    }
    setToolTip(msg);
}
